from abc import ABC, abstractmethod
from dataclasses import dataclass

# unnecessary OOP?
@dataclass
class SchemeData:
    def __init__(self):
        self.points = {}
        self.elements = []
        self.everything = []

        # task data
        self.point = None
        self.time = None


class Something(ABC):
    def get_value(self, data: SchemeData) -> bool:
        if self.current_value is None:
            self.set_next_value(data)
            self.update()
        
        return self.current_value

    def get_next_value(self, data: SchemeData) -> bool:
        return self.current_value if self.next_value is None else self.next_value

    @abstractmethod
    def set_next_value(self, data: SchemeData) -> None:
        pass

    def update(self) -> None:
        self.current_value = self.next_value

    def __str__(self):
        return str(int(self.current_value))
        

class Point(Something):
    def __init__(self, point_id: int, value: int):
        self.point_id = point_id
        self.current_value = value
        self.next_value = None

    def set_next_value(self, data: SchemeData) -> None:
        res = False
        has_connections = False
        
        for elem in data.elements:
            if elem.out_id != self.point_id:
                continue

            has_connections = True
            
            if elem.get_value(data):
                res = True

        if not has_connections:
            self.next_value = self.current_value
        else:
            self.next_value = res

    def debug_data(self) -> str:
        return f"Point {self.point_id}: {int(self.current_value)} {int(self.next_value)}"


class ElementNOT(Something):
    def __init__(self, in_id: int, out_id: int):
        self.in_id = in_id
        self.out_id = out_id
        self.current_value = None
        self.next_value = None

    def set_next_value(self, data: SchemeData) -> None:
        self.next_value = (not data.points[self.in_id].get_next_value(data))

    def debug_data(self) -> str:
        return f"NOT {self.in_id} {self.out_id}: {int(self.current_value)} {int(self.next_value)}"


class ElementAND(Something):
    def __init__(self, in1_id: int, in2_id: int, out_id: int):
        self.in1_id = in1_id
        self.in2_id = in2_id
        self.out_id = out_id
        self.current_value = None
        self.next_value = None

    def set_next_value(self, data: SchemeData) -> None:
        self.next_value = data.points[self.in1_id].get_next_value(data)\
        and data.points[self.in2_id].get_next_value(data)

    def debug_data(self) -> str:
        return f"AND {self.in1_id} {self.in2_id} {self.out_id}: {int(self.current_value)} {int(self.next_value)}"


# unnecessary pattern strategy?
class DescriptionStrategy(ABC):
    @abstractmethod
    def run(self, data: SchemeData, args: list) -> None:
        pass


class NewPoint(DescriptionStrategy):
    def run(self, data: SchemeData, args: list) -> None:
        p = Point(args[0], args[1])
        data.points[args[0]] = p
        data.everything.append(p)


class NewNOT(DescriptionStrategy):
    def run(self, data: SchemeData, args: list) -> None:
        e = ElementNOT(args[0], args[1])
        data.elements.append(e)
        data.everything.append(e)


class NewAND(DescriptionStrategy):
    def run(self, data: SchemeData, args: list) -> None:
        e = ElementAND(args[0], args[1], args[2])
        data.elements.append(e)
        data.everything.append(e)


class TaskData(DescriptionStrategy):
    def run(self, data: SchemeData, args: list) -> None:
        data.point = args[0]
        data.time = args[1]

strategies = {
    0: NewPoint(),
    1: NewNOT(),
    2: NewAND(),
    3: TaskData(),
}

data = SchemeData()

# этот код по-моему ещё и нерабочий, хотя с ответом первого теста сходится

n = int(input())
for i in range(n):
    args = list(map(int, input().split()))
    strategies[args[0]].run(data, args[1:])

for i in range(data.time):
    for point in data.points.values():
        point.set_next_value(data)

    for element in data.elements:
        element.set_next_value(data)

    # debug
    # for smth in data.everything:
    #     print(smth.debug_data())
    # print("--------------")

    for smth in data.everything:
        smth.update()

    print(data.points[data.point])
